import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Map.Entry;
import javax.xml.ws.Holder;

import org.biojava3.alignment.Alignments;
import org.biojava3.alignment.Alignments.PairwiseSequenceAlignerType;
import org.biojava3.alignment.SimpleGapPenalty;
import org.biojava3.alignment.SimpleSubstitutionMatrix;
import org.biojava3.alignment.template.SequencePair;
import org.biojava3.alignment.template.SubstitutionMatrix;
import org.biojava3.core.sequence.ProteinSequence;
import org.biojava3.core.sequence.compound.AminoAcidCompound;

import nl.ru.cmbi.mrs.clustal.MrswsClustal;
import nl.ru.cmbi.mrs.clustal.MrswsClustalPortType;
import nl.ru.cmbi.mrs.clustal.Sequence;

/**
 * Make a MSA by Clustal of given sequences.
 * @author Pawel P. Wozniak
 */
public class AlignmentMakerUniversal 
{
	private List<String> alignment = new ArrayList<String>();
		
	/**
	 * @return list of sequences after alignment
	 */
	public List<String> getAlignment()
	{
		List<String> copy = new ArrayList<String>(alignment.size());
	    for (String c : alignment)
	    {
	        copy.add(c);
	    }
		return copy;
	}

	/**
	 * @param sequences - list of sequences to align
	 */
	public AlignmentMakerUniversal(List<String> sequences) 
	{
		if(sequences.size() == 2)
		{
			biojavaAlignment(sequences);
		}
		else
		{
			clustalAlignment(sequences);
		}
	}
	
	/**
	 * Create pairwise alignment with biojava.
	 * @param sequences - list of sequences to align
	 */
	private void biojavaAlignment(List<String> sequences)
	{
		SubstitutionMatrix<AminoAcidCompound> matrix = new SimpleSubstitutionMatrix<AminoAcidCompound>();
		ProteinSequence s1 = new ProteinSequence(sequences.get(0));
		ProteinSequence s2 = new ProteinSequence(sequences.get(1));
        SequencePair<ProteinSequence, AminoAcidCompound> pair = Alignments.getPairwiseAlignment(s1, s2, PairwiseSequenceAlignerType.GLOBAL, new SimpleGapPenalty(), matrix);
        alignment.add(pair.getQuery().getSequenceAsString());
        alignment.add(pair.getTarget().getSequenceAsString());
	}
	
	/**
	 * Create alignment with clustalW.
	 * @param sequences - list of sequences to align
	 */
	private void clustalAlignment(List<String> sequences)
	{
		MrswsClustalPortType port = new MrswsClustal().getMrswsClustal();
		
		HashMap<String, String> seqs = new HashMap<String, String>();
		
		int SeqNmb = sequences.size();
		for(int i=0; i<SeqNmb; i++)
		{
			seqs.put(String.valueOf(i), sequences.get(i));
		}
		
		StringBuilder sbSq = new StringBuilder();
		List<Sequence> input = new ArrayList<Sequence>();
		Map<String, String> acSequences = seqs;
		for (Entry<String, String> entry : acSequences.entrySet()) {
			Sequence seq = new Sequence();
			seq.setId(entry.getKey());
			seq.setSequence(entry.getValue());
			input.add(seq);
			//Debug toStringBuilder
			sbSq.append('\n').append(seq.getId());
			sbSq.append('\t').append(seq.getSequence());
		}

		//Create holders
		Holder<String> stdOut = new Holder<String>();
		Holder<String> stdErr = new Holder<String>();
		Holder<List<Sequence>> output = new Holder<List<Sequence>>();

		//Invoke clustalW
		port.clustalW(input, 5, 2.5f, stdOut, stdErr, output);

		//Retrieve output
		StringBuilder sbAl = new StringBuilder();
		SortedMap<String, String> acAlignments = new TreeMap<String, String>();
		for (Sequence al : output.value) {
			acAlignments.put(al.getId(), al.getSequence());
			//Debug toStringBuilder
			sbAl.append('\n').append(al.getId());
			sbAl.append('\t').append(al.getSequence());
		}
		
		for(int i=0; i<SeqNmb; i++)
		{
			alignment.add(acAlignments.get(String.valueOf(i)));
		}
	}
}
