import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * Class gets the fasta sequences out of the fasta file.
 * @author Pawel P. Wozniak
 */
public class FastaFileReader
{
	private List<String> fastaSeqs = new ArrayList<String>();
	private List<String> fastaTitles = new ArrayList<String>();
	
	/**
	 * Get fasta sequences.
	 * @return list with fasta sequences
	 */
	public List<String> getFastaSeqs()
	{
		List<String> copy = new ArrayList<String>(fastaSeqs.size());
	    for (String c : fastaSeqs)
	    {
	        copy.add(c);
	    }
		return copy;
	}
	
	/**
	 * Get fasta sequences' titles.
	 * @return list with fasta sequences' titles
	 */
	public List<String> getFastaTitles()
	{
		List<String> copy = new ArrayList<String>(fastaTitles.size());
	    for (String c : fastaTitles)
	    {
	        copy.add(c);
	    }
		return copy;
	}

	/**
	 * Get top sequences from Fasta file. !!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 * @param fastaPath - path to the fasta file
	 */
	FastaFileReader(String fastaPath, int howMany)
	{
		try
		{
			FileReader fr = new FileReader(fastaPath);
			BufferedReader br = new BufferedReader(fr);
			String line;
			int counter = 0;
			boolean startSeq = false;
			String tmpSeq = "";
			while((line = br.readLine()) != null && counter <= howMany)
			{
				if(line.startsWith(">"))
				{
					fastaTitles.add(line);
					if(!tmpSeq.isEmpty())
					{
						fastaSeqs.add(tmpSeq);
					}
					tmpSeq = "";
					startSeq = true;
					counter++;
				}
				else if(startSeq)
				{
					tmpSeq = tmpSeq + line.trim();
				}
			}
			br.close();
			fr.close();
			
			if(!tmpSeq.isEmpty())
			{
				fastaSeqs.add(tmpSeq);
			}
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Get all sequences from Fasta file.
	 * @param fastaPath - path to the fasta file
	 */
	FastaFileReader(String fastaPath)
	{
		try
		{
			ReadFile rf = new ReadFile(fastaPath);
			String inFile = rf.getFullText();
			Pattern pattern = Pattern.compile(">.*\\n(([^>]|[.\\s])+)");
			Matcher matcher = pattern.matcher(inFile);
			String match;
			while(matcher.find())
			{
				match = matcher.group(1);
			    inFile = inFile.replaceFirst(match, "");
			    matcher = pattern.matcher(inFile);
			    match = match.replaceAll("(\\s|\\n|\\t|\\r)", "");
				fastaSeqs.add(match);
			}
			
			if(fastaSeqs.isEmpty())
			{
				pattern = Pattern.compile(">.*\\r\\n(([^>]|[.\\s])+)");
				matcher = pattern.matcher(inFile);
				while(matcher.find())
				{
					match = matcher.group(1);
				    inFile = inFile.replaceFirst(match, "");
				    matcher = pattern.matcher(inFile);
				    match = match.replaceAll("(\\s|\\n|\\t|\\r)", "");
					fastaSeqs.add(match);
				}
			}
			
			if(!fastaSeqs.isEmpty())
			{
				String[] splitter = inFile.split("\\r\\n");
				for (int i = 0; i < splitter.length; i++)
				{
					fastaTitles.add(splitter[i].trim());
				}
			}
			
			if(fastaSeqs.size() == fastaTitles.size())
			{
				System.out.println(fastaSeqs.size() + " seqs");
			}
			else
			{
				System.out.println("Number of sequences does not equal number of titles!!!");
				fastaSeqs = null;
				fastaTitles = null;
			}
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
}
