import java.io.FileWriter;
import java.util.List;

/**
 * Create a file with parameters calculated for single protein. Parameters are used later by R-script models to predict PPV of DCA.
 * If you use any part of this script (and any other from this package) please cite our work:
 * PPWozniak, BM Konopka, GVriend, MKotulska: "Forecasting residue-residue contact prediction accuracy"
 * @author Pawel P. Wozniak
 */
public class Main_CreateListWithParams_SINGLE
{
	/**
	 * @param args - table with: 
	 * 1) path to DCA results file; 
	 * 2) path to SCRATCH .ss file; 
	 * 3) path to SCRATCH .acc20 file; 
	 * 4) path to FASTA file; 
	 * 5) path to MSA file; 
	 * 6) true - if each line in the MSA file is a separate sequence (no FASTA headers) - false otherwise; 
	 * 7) separation between two amino acids in sequence to treat them as contacts; 
	 * 8) number of strongest predicted contacts in DCA results file to calculate parameters for; 
	 * 9) number of column in DCA result file with sequence number of first amino acid in contact (start with 0); 
	 * 10) number of column in DCA result file with sequence number of second amino acid in contact (start with 0); 
	 * 11) number of column in DCA result file with DCA result; 
	 * 12) separator of columns in DCA file;
	 */
	public static void main(String[] args)
	{	
		String resFilePath = args[0];
		String ssFilePath = args[1];
		String accFilePath = args[2];
		String fastaFilePath = args[3];
		String msaFilePath = args[4];
		
		boolean ifEachLineMSA = Boolean.parseBoolean(args[5]);
		int separation = Integer.parseInt(args[6]);
		int top = Integer.parseInt(args[7]);
		int pos1Col = Integer.parseInt(args[8]);
		int pos2Col = Integer.parseInt(args[9]);
		int dcaCol = Integer.parseInt(args[10]);
		String splitRegex = args[11];
		
		//String resFilePath = "1Y60C.gplmdca";
		//String ssFilePath = "1Y60C.out.ss";
		//String accFilePath = "1Y60C.out.acc20";
		//String fastaFilePath = "1Y60C.fasta";
		//String msaFilePath = "1Y60C.a3m";
		
		//boolean ifEachLineMSA = false;
		//int separation = 5;
		//int top = 200;
		//int pos1Col = 0;
		//int pos2Col = 1;
		//int dcaCol = 2;
		//String splitRegex = ",";
		
		try
		{
			FileWriter writer = new FileWriter(resFilePath.replaceAll("\\.", "_") + "_top" + top);
			String[] titles = CreateListWithParams.getTitles();
			writer.write("NAME ");
			for (int i = 0; i < titles.length; i++) {
				writer.write(titles[i] + " ");
			}
			writer.write("\n");
			CreateListWithParams listObject = new CreateListWithParams(resFilePath, fastaFilePath, 
					msaFilePath, ssFilePath, accFilePath, ifEachLineMSA, separation, top, pos1Col, 
					pos2Col, dcaCol, splitRegex);
			List<Double> params = listObject.getParams();
			writer.write(resFilePath.replaceAll("\\.", "_") + " ");
			for (int j = 0; j < titles.length; j++) {
				writer.write(params.get(j) + " ");
			}
			writer.write("\n");
			writer.close();
		}catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
}
