import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Parse SCRATCH .ss and .acc20 files.
 * @author Pawel P. Wozniak
 */
public class ScratchReader
{
	/**
	 * Read SSpro result for chosen protein in file with results
	 * @param filePath - path to the file with SSpro results
	 * @param titleContain - code of the chosen protein to find in the file with results (if null then first found)
	 * @return list of SS symbols (H,E,C) predicted for each AA in the chosen protein
	 */
	public static List<Character> ssRead(String filePath, String titleContain)
	{
		List<Character> ssSeq = new ArrayList<Character>();
		try
		{
			FileReader fr = new FileReader(filePath);
			BufferedReader br = new BufferedReader(fr);
			String line;
			while((line=br.readLine()) != null)
			{
				if(line.startsWith(">") && (titleContain == null || line.contains(titleContain)))
				{
					line=br.readLine();
					for (int i = 0; i < line.length(); i++)
					{
						ssSeq.add(line.charAt(i));
					}
				}
			}
			br.close();
			fr.close();
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			return null;
		}
		return ssSeq;
	}
	
	/**
	 * Read ACCpro result for chosen protein in file with results
	 * @param filePath - path to the file with ACCpro results
	 * @param titleContain - code of the chosen protein to find in the file with results (if null then first found)
	 * @return list of RSA values (Relative Solvent Accessibility) predicted for each AA in the chosen protein
	 */
	public static List<Integer> accRead(String filePath, String titleContain)
	{
		List<Integer> accSeq = new ArrayList<Integer>();
		try
		{
			FileReader fr = new FileReader(filePath);
			BufferedReader br = new BufferedReader(fr);
			String line;
			while((line=br.readLine()) != null)
			{
				if(line.startsWith(">") && (titleContain == null || line.contains(titleContain)))
				{
					line=br.readLine();
					String[] splitter = line.split("\\s+");
					for (int i = 0; i < splitter.length; i++)
					{
						accSeq.add(Integer.parseInt(splitter[i]));
					}
				}
			}
			br.close();
			fr.close();
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			return null;
		}
		return accSeq;
	}
}
